#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <cmath>
#include <iostream>

unsigned MainWindow::s_maxNbOfPairs = 32;
unsigned MainWindow::s_hideCardsTimeout = 2000;
QSize MainWindow::s_cardFixedSize= QSize(40, 80); //this is how statics are initialized. You'll never remember this

MainWindow::MainWindow(QWidget *parent) :
	QMainWindow(parent),
	ui(new Ui::MainWindow),
	m_memory(),
	m_cardButtons(),
	m_selectedCards(),
	m_timerHideCards(nullptr)
{
	m_selectedCards[0]=nullptr;
	m_selectedCards[1]=nullptr;
	m_timerHideCards = new QTimer(this);
	connect(m_timerHideCards, SIGNAL(timeout()), this, SLOT(on_timerHideCards_timeout()));
	ui->setupUi(this);
	ui->spinBox_nbPairs->setMaximum(s_maxNbOfPairs);
	on_pushButton_reset_pressed();
}

MainWindow::~MainWindow()
{
	delete ui;
	for(CardButton *button : m_cardButtons)
		delete button;
}

void MainWindow::refreshMemoryCards()
{
	m_memory.setNbCards(ui->spinBox_nbPairs->value()*2);
	ui->label_score->setText(QString::number(m_memory.score()));
	m_memory.shuffle();
	m_cardButtons.resize(m_memory.nbCards());
	m_selectedCards[0] = nullptr;
	m_selectedCards[1] = nullptr;
	for(unsigned i=0; i<m_memory.nbCards(); ++i)
	{
		CardButton* &cardButton = m_cardButtons[i];
		const Card &card = m_memory.getCard(i);
		cardButton = new CardButton(this, i);
		cardButton->setFixedSize(s_cardFixedSize);
		cardButton->setText(card.revealed	? QString::fromStdString(std::to_string(card.patternId)) //one way
											: QString());
		QSize index=_unsignedToQSize(i);
		ui->gridLayout_cards->addWidget(cardButton, index.width(), index.height());
		QObject::connect(cardButton, SIGNAL(released()), this, SLOT(on_cardButton_pushed()));
	}
}

QSize MainWindow::_unsignedToQSize(unsigned index)
{
	QSize size;
	size.setWidth(index/_GetNbColumnsLayout());
	size.setHeight(index%_GetNbColumnsLayout());
	return size;
}

unsigned MainWindow::_QSizeToUnsigned(QSize size)
{
	return unsigned(size.width()*_GetNbColumnsLayout() + size.height());
}

unsigned MainWindow::_GetNbColumnsLayout()
{
	return unsigned(std::ceil(std::sqrt(m_memory.nbCards())));
}

void MainWindow::on_cardButton_pushed()
{
	if(m_selectedCards[1]==nullptr)
	{
		QPushButton* pushButtonSender = qobject_cast<QPushButton*>(sender()); //retrieve the button you have pushed
		CardButton* cardButtonSender = dynamic_cast<CardButton*>(pushButtonSender);
		bool pairFound = m_memory.revealCard(cardButtonSender->cardPosition());
		ui->label_score->setText(QString::number(m_memory.score()));
		const Card& card = m_memory.getCard(cardButtonSender->cardPosition());
		cardButtonSender->setText(card.revealed	? QString::number(card.patternId) //another way
												: QString());
		if(m_selectedCards[0]==nullptr)
		{
			m_selectedCards[0]=cardButtonSender;
		}
		else
		{
			m_selectedCards[1]=cardButtonSender;
			if(pairFound)
			{
				m_selectedCards[0]->setEnabled(false);
				m_selectedCards[1]->setEnabled(false);
				m_selectedCards[0]=nullptr;
				m_selectedCards[1]=nullptr;
				m_memory.hideRevealedCards();
				if(m_memory.checkWinCondition())
				{
					int doSomethingWhatever;
				}
			}
			else
			{
				m_timerHideCards->start(s_hideCardsTimeout);
			}
		}
	}
}

void MainWindow::on_timerHideCards_timeout()
{
	m_timerHideCards->stop();
	m_memory.hideRevealedCards();
	const Card& card0 = m_memory.getCard(m_selectedCards[0]->cardPosition());
	m_selectedCards[0]->setText(card0.revealed	? QString::number(card0.patternId) //another way
												: QString());
	m_selectedCards[0] = nullptr;
	const Card& card1 = m_memory.getCard(m_selectedCards[1]->cardPosition());
	m_selectedCards[1]->setText(card1.revealed	? QString::number(card1.patternId) //another way
												: QString());
	m_selectedCards[1] = nullptr;
}

void MainWindow::on_pushButton_play_pressed()
{
	m_timerHideCards->stop();
	ui->pushButton_play->setHidden(true);
	for(CardButton* &button : m_cardButtons)
	{
		button->setEnabled(true);
	}
}

void MainWindow::on_pushButton_reset_pressed()
{
	m_timerHideCards->stop();
	clearLayout(ui->gridLayout_cards);
	refreshMemoryCards();
	for(CardButton *button : m_cardButtons)
		button->setEnabled(false);
	ui->pushButton_play->setHidden(false);
}

void clearLayout(QLayout *layout)
{
	QLayoutItem *item;
	while((item = layout->takeAt(0)))
	{
		if (item->layout()!=nullptr)
		{
			clearLayout(item->layout());
			delete item->layout();
		}
		if (item->widget()!=nullptr)
		{
		   delete item->widget();
		}
		delete item;
	}
}
