#include "memory.h"

Card::Card(unsigned patternId):
	revealed(false),
	pairFound(false),
	patternId(patternId)
{}

Memory::Memory() :
	m_cards(),
	m_turnedCard(),
	m_score(0)
{
	m_turnedCard[0] = nullptr;
	m_turnedCard[1] = nullptr;
}

void Memory::setNbCards(unsigned nbCards)
{
	assert(nbCards%2 == 0);
	m_cards.resize(nbCards);
	unsigned id=0;
	m_score = 0;
	for(Card &card : m_cards)
	{
		card.patternId = id++/2;
		card.pairFound = false;
		card.revealed = false;
	}
	m_turnedCard[0] = nullptr;
	m_turnedCard[1] = nullptr;
}

unsigned Memory::nbCards() const
{
	return m_cards.size();
}

int Memory::score() const
{
	return m_score;
}

const Card& Memory::getCard(unsigned cardNb) const
{
	assert(m_cards.size()>cardNb);
	return m_cards[cardNb];
}

bool Memory::revealCard(unsigned cardNb)
{
	assert(m_cards.size()>cardNb);
	bool newPairFound = false;
	if(m_turnedCard[0] != nullptr)
	{
		if(m_turnedCard[1]==nullptr) //else do nothing I guess
		{
			m_turnedCard[1] = &m_cards[cardNb];
			assert(m_turnedCard[0] != m_turnedCard[1]);
			assert(!m_turnedCard[1]->pairFound);
			if(m_turnedCard[0]->patternId == m_turnedCard[1]->patternId)
			{
				newPairFound = true;
				m_turnedCard[0]->pairFound = true;
				m_turnedCard[1]->pairFound = true;
				m_score += 10;
			}
			else
				m_score -= 1;
			m_turnedCard[1]->revealed = true;
		}
	}
	else
	{
		m_turnedCard[0] = &m_cards[cardNb];
		assert(!m_turnedCard[0]->pairFound);
		m_turnedCard[0]->revealed = true;
	}
	return newPairFound;
}

void Memory::hideRevealedCards()
{
	if(m_turnedCard[0]!=nullptr)
	{
		m_turnedCard[0]->revealed = m_turnedCard[0]->pairFound ? true : false;
		m_turnedCard[0] = nullptr;
	}
	if(m_turnedCard[1]!=nullptr)
	{
		m_turnedCard[1]->revealed = m_turnedCard[1]->pairFound ? true : false;
		m_turnedCard[1] = nullptr;
	}
}

void Memory::shuffle()
{
	std::random_shuffle(m_cards.begin(), m_cards.end()-1);
}

bool Memory::checkWinCondition()
{
	bool isWin = true;
	for(unsigned i=0; isWin && i<nbCards(); ++i)
	{
		if(!getCard(i).pairFound)
			isWin=false;
	}
	return isWin;
}
