#ifndef MEMORY_H
#define MEMORY_H

#include <vector>
#include <algorithm>
#include <assert.h>
#include <cstdlib>

//You can have classes in C++! This is the declaration of the class that is yet to implement.
class Card
{
public:
	Card(unsigned patternId=0); //This is a constructor with a default argument.

	bool revealed;
	bool pairFound;
	unsigned patternId;
};


class Memory
{
public:
	Memory();	//This is a constructor. It initializes every variable in the class.
				//It does not allocate memory for the class, but it can allocate memory for its members
				//(new Memory() does though, as well as declaring a Memory object in the stack)

	void setNbCards(unsigned nbCards);	//This is a public member function. It can be called on an instance of Memory.
	unsigned nbCards() const;
	int score() const;

	const Card& getCard(unsigned cardNb) const;  //There is a const reference. It's like a const pointer, but it's never null!
	bool revealCard(unsigned cardNb);
	bool canRevealCards() const;
	void hideRevealedCards();
	void shuffle();
	bool checkWinCondition();

private:
	std::vector<Card> m_cards;	//This is a private class member. std::vector is the most common convenient container
									//(check en.cppreference.com/w/cpp/container/vector for a list of its functions).
									//<card_t> is its first template argument, making it a vector of card_t.
									//Vector is used just like an array (yes, you can use operator []), but it
									//also keeps track of the size, permits easy bilateral iterations, and allows
									//easy insertions.
	Card *m_turnedCard[2];
	int m_score;
};

#endif // MEMORY_H
